#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# IMPORTS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

import unittest;
from unittest import TestCase;

from aussagenlogik.schema import string_to_parts;
from aussagenlogik.rekursion import rekursiv_atoms;
from aussagenlogik.rekursion import rekursiv_depth;
from aussagenlogik.rekursion import rekursiv_length;
from aussagenlogik.rekursion import rekursiv_parentheses;
from aussagenlogik.rekursion import rekursiv_eval;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# GLOBALE KONSTANTEN
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TESTFALL Atome(·)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@unittest.skip('Methode noch nicht implementiert')
class TestRekursivAtoms(TestCase):
    def test_noduplicates(self):
        fml = string_to_parts('( A4 && ( A4 || A4 ))');
        val = sorted(rekursiv_atoms(fml));
        assert len([_ for _ in val if _ == 'A4']) == 1, 'Atome dürfen nicht mehrfach vorkommen!';

    def test_nononatoms(self):
        fml = string_to_parts('( {F} || A3 )');
        val = sorted(rekursiv_atoms(fml));
        assert 'F' not in val, 'Nichtatomare Formeln dürfen nicht vorkommen!';

    def test_calc1(self):
        fml = string_to_parts('A0');
        val = sorted(rekursiv_atoms(fml));
        assert val == ['A0'], 'computed {}'.format(val);

    def test_calc2(self):
        fml = string_to_parts('((( ! A0 && A3 ) || A4 ) && A8 )');
        val = sorted(rekursiv_atoms(fml));
        assert val == ['A0', 'A3', 'A4', 'A8'], 'computed {}'.format(val);
    pass;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TESTFALL Depth(·)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@unittest.skip('Methode noch nicht implementiert')
class TestRekursivDepth(TestCase):
    def test_calc1(self):
        fml = string_to_parts('A0');
        val = rekursiv_depth(fml);
        assert val == 0, 'computed {}'.format(val);

    def test_calc2(self):
        fml = string_to_parts('!! A8');
        val = rekursiv_depth(fml);
        assert val == 2, 'computed {}'.format(val);

    def test_calc3(self):
        fml = string_to_parts('( ! A0 && A3 )');
        val = rekursiv_depth(fml);
        assert val == 2, 'computed {}'.format(val);

    def test_calc4(self):
        fml = string_to_parts('((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_depth(fml);
        assert val == 4, 'computed {}'.format(val);

    def test_calc5(self):
        fml = string_to_parts('! ((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_depth(fml);
        assert val == 5, 'computed {}'.format(val);
    pass;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TESTFALL Länge(·)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@unittest.skip('Methode noch nicht implementiert')
class TestRekursivLength(TestCase):
    def test_calc1(self):
        fml = string_to_parts('A0');
        val = rekursiv_length(fml);
        assert val == 1, 'computed {}'.format(val);

    def test_calc2(self):
        fml = string_to_parts('!! A8');
        val = rekursiv_length(fml);
        assert val == 3, 'computed {}'.format(val);

    def test_calc3(self):
        fml = string_to_parts('( ! A0 && A3 )');
        val = rekursiv_length(fml);
        assert val == 4, 'computed {}'.format(val);

    def test_calc4(self):
        fml = string_to_parts('((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_length(fml);
        assert val == 8, 'computed {}'.format(val);

    def test_calc5(self):
        fml = string_to_parts('! ((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_length(fml);
        assert val == 9, 'computed {}'.format(val);
    pass;


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TESTFALL Anzahl Klammern(·)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

@unittest.skip('Methode noch nicht implementiert')
class TestRekursivParentheses(TestCase):
    def test_calc1(self):
        fml = string_to_parts('A0');
        val = rekursiv_parentheses(fml);
        assert val == 0, 'computed {}'.format(val);

    def test_calc2(self):
        fml = string_to_parts('!! A8');
        val = rekursiv_parentheses(fml);
        assert val == 0, 'computed {}'.format(val);

    def test_calc3(self):
        fml = string_to_parts('( ! A0 && A3 )');
        val = rekursiv_parentheses(fml);
        assert val == 2, 'computed {}'.format(val);

    def test_calc4(self):
        fml = string_to_parts('((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_parentheses(fml);
        assert val == 6, 'computed {}'.format(val);

    def test_calc5(self):
        fml = string_to_parts('! ((( ! A0 && A3 ) || A4 ) && A8 )');
        val = rekursiv_parentheses(fml);
        assert val == 6, 'computed {}'.format(val);
    pass;


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# TESTFALL eval(·, ·)
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# @unittest.skip('Methode noch nicht implementiert')
class TestRekursivEval(TestCase):
    def test_literale(self):
        fml = string_to_parts('A0');
        val = rekursiv_eval(fml, [ 'A0' ]);
        assert val == 1;
        fml = string_to_parts('A0');
        val = rekursiv_eval(fml, []);
        assert val == 0;
        fml = string_to_parts('! A0');
        val = rekursiv_eval(fml, [ 'A0' ]);
        assert val == 0;
        fml = string_to_parts('! A0');
        val = rekursiv_eval(fml, []);
        assert val == 1;

    def test_complex1(self):
        fml = string_to_parts('( ! A0 || (( A0 && A3 ) || A2 ))');
        val = rekursiv_eval(fml, [ 'A0', 'A2' ]);
        assert val == 1;
        val = rekursiv_eval(fml, [ 'A0', 'A3' ]);
        assert val == 1;
        val = rekursiv_eval(fml, [ 'A0' ]);
        assert val == 0;
        val = rekursiv_eval(fml, [ 'A4', 'A8' ]);
        assert val == 1;

    def test_complex2(self):
        fml = string_to_parts('( ! A0 || (( A0 && A3 ) || ! A2 ))');
        val = rekursiv_eval(fml, [ 'A0', 'A2' ]);
        assert val == 0;
        val = rekursiv_eval(fml, [ 'A0', 'A3' ]);
        assert val == 1;
    pass;
